"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_utils_1 = require("./test-utils");
const properties = require("../properties.json");
describe("Validations", () => {
    let form;
    // Uncomment when https://github.com/rjsf-team/react-jsonschema-form/pull/1528 is merged.
    //beforeAll(async () => {
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        form = yield test_utils_1.createForm();
    }));
    const schema = {
        type: "object",
        properties: {
            a: {
                type: "string"
            },
            b: {
                type: "string"
            }
        },
    };
    const formData = { b: "bar" };
    const message = "foo";
    const getCustom = (message = "foo", live) => ({
        a: {
            presence: {
                message,
                options: {
                    _live: live
                }
            }
        }
    });
    it("pass on no errors", () => __awaiter(void 0, void 0, void 0, function* () {
        yield form.setState({ schema, formData });
        yield form.submit();
        expect(yield form.getSubmittedData()).toEqual(formData);
    }));
    it("fail on schema errors", () => __awaiter(void 0, void 0, void 0, function* () {
        yield form.setState({ schema: Object.assign(Object.assign({}, schema), { required: ["a"] }), formData });
        yield form.submit();
        expect(yield form.getSubmittedData()).not.toEqual(formData);
        expect(yield form.errors.$$all.count()).toBe(1);
    }));
    it("doesn't run schema validators before submitted", () => __awaiter(void 0, void 0, void 0, function* () {
        const _formData = Object.assign(Object.assign({}, formData), { a: "foo" });
        yield form.setState({ schema: Object.assign(Object.assign({}, schema), { required: ["a"] }), formData: _formData });
        yield test_utils_1.updateValue(form.$getInputWidget("a"), "");
        expect(yield form.errors.$$all.count()).toBe(0);
        yield form.submit();
        expect(yield form.errors.$$all.count()).toBe(1);
    }));
    it("fail on custom errors", () => __awaiter(void 0, void 0, void 0, function* () {
        const validators = getCustom();
        const formData = {};
        yield form.setState({ schema, formData, validators });
        yield form.submit();
        expect(yield form.getSubmittedData()).not.toEqual(formData);
        expect(yield form.errors.$$all.get(0).getText()).toContain(message);
    }));
    describe("warnings", () => {
        it("prevent submit", () => __awaiter(void 0, void 0, void 0, function* () {
            const warnings = getCustom();
            yield form.setState({ schema, formData, warnings });
            yield form.submit();
            expect(yield form.getSubmittedData()).not.toEqual(formData);
            expect(yield form.warnings.$$all.get(0).getText()).toContain(message);
        }));
        it("can be submitted via warnings-acknowledged-submit button", () => __awaiter(void 0, void 0, void 0, function* () {
            const warnings = getCustom();
            yield form.setState({ schema, formData, warnings });
            yield form.submit();
            yield test_utils_1.waitUntilBlockingLoaderHides();
            yield form.$acknowledgeWarnings.click();
            expect(yield form.getSubmittedData()).toEqual(formData);
        }));
    });
    [["errors", "validators"], ["warnings", "warnings"]].forEach(([type, propName]) => {
        it(`doesn't run ${type} live validators on init`, () => __awaiter(void 0, void 0, void 0, function* () {
            const validators = getCustom(message, "live");
            const formData = {};
            yield form.setState({ schema, formData, [propName]: validators });
            expect(yield form[type].$panel.isPresent()).toBe(false);
        }));
        it(`runs live ${type} validators on onChange`, () => __awaiter(void 0, void 0, void 0, function* () {
            const validators = getCustom(message, "live");
            const formData = { a: "bar" };
            yield form.setState({ schema, formData, [propName]: validators });
            yield test_utils_1.updateValue(form.$getInputWidget("a"), "");
            expect(yield form[type].$$all.get(0).getText()).toContain(message);
        }));
    });
    it("doesn't show blocker during live validation", () => __awaiter(void 0, void 0, void 0, function* () {
        const validators = getCustom(message, "live");
        const formData = { a: "bar" };
        yield form.setState({ schema, formData, validators });
        yield test_utils_1.updateValue(form.$getInputWidget("a"), "");
        expect(yield form.isBlocked()).toBe(false);
    }));
    it("runs both validators and warnings live on Change", () => __awaiter(void 0, void 0, void 0, function* () {
        const validators = getCustom(message, "live");
        const formData = { a: "bar" };
        yield form.setState({ schema, formData, validators, warnings: validators });
        yield test_utils_1.updateValue(form.$getInputWidget("a"), "");
        for (const type of ["errors", "warnings"]) {
            expect(yield form[type].$$all.get(0).getText()).toContain(message);
        }
    }));
    it("keeps non live errors when doing only live validation", () => __awaiter(void 0, void 0, void 0, function* () {
        const validators = getCustom(message, "live");
        const warnings = getCustom(message);
        const formData = { a: "bar" };
        yield form.setState({ schema, formData, validators, warnings });
        yield test_utils_1.updateValue(form.$getInputWidget("a"), "");
        yield form.submit();
        yield test_utils_1.waitUntilBlockingLoaderHides();
        expect(yield form.warnings.$$all.get(0).getText()).toContain(message);
        expect(yield form.errors.$$all.get(0).getText()).toContain(message);
        yield test_utils_1.updateValue(form.$getInputWidget("a"), "bar");
        expect(yield form.warnings.$$all.get(0).getText()).toContain(message);
        expect(yield form.errors.$$all.count()).toBe(0);
        yield test_utils_1.updateValue(form.$getInputWidget("a"), "");
        for (const type of ["errors", "warnings"]) {
            expect(yield form[type].$$all.get(0).getText()).toContain(message);
        }
    }));
    it("cached non live errors are cleared when validated again against valid", () => __awaiter(void 0, void 0, void 0, function* () {
        const validators = getCustom(message, "live");
        const warnings = getCustom(message);
        const formData = { a: "bar" };
        yield form.setState({ schema, formData, validators, warnings });
        yield test_utils_1.updateValue(form.$getInputWidget("a"), "");
        yield form.submit();
        yield test_utils_1.waitUntilBlockingLoaderHides();
        expect(yield form.warnings.$$all.get(0).getText()).toContain(message);
        expect(yield form.errors.$$all.get(0).getText()).toContain(message);
        yield test_utils_1.updateValue(form.$getInputWidget("a"), "bar");
        expect(yield form.warnings.$$all.get(0).getText()).toContain(message);
        expect(yield form.errors.$$all.count()).toBe(0);
        yield test_utils_1.updateValue(form.$getInputWidget("a"), "");
        yield test_utils_1.updateValue(form.$getInputWidget("a"), "bar");
        yield form.submit();
        for (const type of ["errors", "warnings"]) {
            expect(yield form[type].$$all.count()).toBe(0);
        }
    }));
    it("runs all validator types on submit", () => __awaiter(void 0, void 0, void 0, function* () {
        const validators = getCustom(message, "live");
        const _validators = Object.assign(Object.assign({}, validators), { b: validators.a });
        const warnings = Object.assign(Object.assign({}, validators), { b: validators.a });
        yield form.setState({ schema, formData, validators: _validators, warnings });
        yield test_utils_1.updateValue(form.$getInputWidget("b"), "");
        yield form.submit();
        expect(yield form.getSubmittedData()).not.toEqual(formData);
        expect(yield form.errors.$$all.count()).toBe(2);
        expect(yield form.warnings.$$all.count()).toBe(2);
    }));
    const response = {
        "error": {
            "statusCode": 422,
            "name": "Error",
            "message": "Unprocessable Entity",
            "details": {
                ".a": [
                    "mock"
                ]
            }
        }
    };
    it("delays submit until async validators complete and doesn't submit when invalid", () => __awaiter(void 0, void 0, void 0, function* () {
        const validators = {
            a: {
                remote: {
                    validator: "mock",
                    message
                }
            }
        };
        const { resolve, remove } = yield form.setMockResponse("/documents/validate", false);
        const _formData = Object.assign(Object.assign({}, formData), { a: "empty isn't async validated so we have value here" });
        yield form.setState({ schema, formData: _formData, validators });
        yield form.startSubmit();
        expect(yield form.getSubmittedData()).not.toEqual(_formData);
        expect(yield form.errors.$$all.count()).toBe(0);
        yield resolve({ status: 422, json: response }, !!"raw");
        expect(yield form.errors.$$all.count()).toBe(1);
        expect(yield form.errors.$$all.get(0).getText()).toContain(message);
        yield remove();
    }));
    it("delays submit until async validators complete and submits when valid", () => __awaiter(void 0, void 0, void 0, function* () {
        const validators = {
            a: {
                remote: {
                    validator: "mock",
                    message
                }
            }
        };
        const { resolve, remove } = yield form.setMockResponse("/documents/validate", false);
        const _formData = Object.assign(Object.assign({}, formData), { a: "empty isn't async validated so we have value here" });
        yield form.setState({ schema, formData: _formData, validators });
        yield form.startSubmit();
        expect(yield form.getSubmittedData()).not.toEqual(_formData);
        expect(yield form.errors.$$all.count()).toBe(0);
        yield resolve({});
        expect(yield form.getSubmittedData()).toEqual(_formData);
        yield remove();
    }));
    it("delays all validators until async operations complete", () => __awaiter(void 0, void 0, void 0, function* () {
        const validators = {
            a: {
                remote: {
                    validator: "mock",
                    message
                }
            }
        };
        const { resolve, remove } = yield form.setMockResponse("/documents/validate", false);
        const _formData = Object.assign(Object.assign({}, formData), { a: "empty isn't async validated so we have value here" });
        yield form.setState({ schema, formData: _formData, validators });
        yield form.startSubmit();
        expect(yield form.getSubmittedData()).not.toEqual(_formData);
        expect(yield form.errors.$$all.count()).toBe(0);
        yield resolve({ status: 422, json: response }, !!"raw");
        expect(yield form.errors.$$all.count()).toBe(1);
        expect(yield form.errors.$$all.get(0).getText()).toContain(message);
        yield remove();
    }));
    const uiSchema = {
        c: {
            "ui:field": "ImageArrayField"
        }
    };
    const uiSchemaContext = {
        creator: properties.userId
    };
    const schemaForAsync = Object.assign(Object.assign({}, schema), { properties: Object.assign(Object.assign({}, schema.properties), { c: {
                type: "array",
                items: {
                    type: "string"
                }
            } }) });
    const imageUploaded = "imgae uploaded";
    const asyncValidators = {
        b: {
            remote: {
                validator: "mock",
                message
            },
            presence: {
                message,
                options: {
                    _live: true
                }
            }
        },
        c: {
            length: {
                maximum: 0,
                message: imageUploaded
            }
        }
    };
    it("validations aren't run when async validation running already", () => __awaiter(void 0, void 0, void 0, function* () {
        const { resolve, remove } = yield form.setMockResponse("/documents/validate", false);
        yield form.setState({ schema: schemaForAsync, formData, validators: asyncValidators, uiSchema, uiSchemaContext });
        const { resolve: imageResolve, remove: imageRemove } = yield form.mockImageUpload("c");
        yield form.startSubmit();
        expect(yield form.errors.$$all.count()).toBe(0);
        yield imageResolve();
        expect(yield form.errors.$$all.count()).toBe(0);
        yield resolve({});
        yield remove();
        yield imageRemove();
    }));
    it("revalidates when data is edited during async validation on submit", () => __awaiter(void 0, void 0, void 0, function* () {
        const { resolve, remove } = yield form.setMockResponse("/documents/validate", false);
        yield form.setState({ schema: schemaForAsync, formData, validators: asyncValidators, uiSchema, uiSchemaContext });
        const { resolve: imageResolve, remove: imageRemove } = yield form.mockImageUpload("c");
        yield form.startSubmit();
        expect(yield form.errors.$$all.count()).toBe(0);
        yield imageResolve();
        yield resolve({});
        yield test_utils_1.waitUntilBlockingLoaderHides();
        expect(yield form.errors.$$all.count()).toBe(1);
        expect(yield form.errors.$$all.get(0).getText()).toContain(imageUploaded);
        yield remove();
        yield imageRemove();
    }));
    it("shows blocking loader when validating", () => __awaiter(void 0, void 0, void 0, function* () {
        const { resolve, remove } = yield form.setMockResponse("/documents/validate", false);
        yield form.setState({ schema, formData, validators: asyncValidators });
        expect(yield form.isBlocked()).toBe(false);
        yield form.startSubmit();
        expect(yield form.isBlocked()).toBe(true);
        yield resolve({ status: 422, json: response }, !!"raw");
        yield test_utils_1.waitUntilBlockingLoaderHides();
        expect(yield form.isBlocked()).toBe(false);
        yield remove();
    }));
    describe("Custom validation bypass", () => {
        it("skips custom errors and warnings", () => __awaiter(void 0, void 0, void 0, function* () {
            const validators = getCustom(message);
            yield form.setState({ schema, formData, validators, warnings: validators });
            yield form.submitOnlySchemaValidations();
            yield form.waitUntilBlockingLoaderHides();
            expect(yield form.errors.$$all.count()).toBe(0);
            expect(yield form.warnings.$$all.count()).toBe(0);
        }));
        it("skips live errors and live warnings", () => __awaiter(void 0, void 0, void 0, function* () {
            const validators = getCustom(message, "live");
            yield form.setState({ schema, formData, validators, warnings: validators });
            yield form.submitOnlySchemaValidations();
            yield form.waitUntilBlockingLoaderHides();
            expect(yield form.errors.$$all.count()).toBe(0);
            expect(yield form.warnings.$$all.count()).toBe(0);
        }));
        it("runs schema validations", () => __awaiter(void 0, void 0, void 0, function* () {
            form = yield test_utils_1.createForm();
            yield form.setState({ schema: Object.assign(Object.assign({}, schema), { required: ["a"] }), formData });
            yield form.submitOnlySchemaValidations();
            yield form.waitUntilBlockingLoaderHides();
            expect(yield form.errors.$$all.count()).toBe(1);
        }));
    });
});
